#A script visualizing the relation of the ML models test MAE in depencende of the used training data set size
#- Robin Hilgers, 2023
#Published on Zenodo
#--------------------------------------------------------------------------------
# Copyright (c) 2023 Peter Grünberg Institut, Forschungszentrum Jülich, Germany
# This file is available as free software under the conditions
# of the MIT license as expressed in the LICENSE file in more detail.
#--------------------------------------------------------------------------------

#Imports
from xgboost import XGBRegressor
import numpy as np 
import pandas as pd 
from sklearn.model_selection import  train_test_split
from sklearn.metrics import  mean_absolute_error
from sklearn.preprocessing import StandardScaler
import seaborn as sns
import matplotlib.pyplot as plt

#Random seed
randomSeed=42

#Load data from
filename="../Data/dataInp.txt"
firstBatch=4300
dfPand=pd.read_csv(filename).iloc[:firstBatch]

#Train the model to provided data and return MAE values
def trainBatch(x,y):
    trainX,testX,trainY,testY=train_test_split(x,y,test_size=0.2, shuffle=True ,random_state=randomSeed)
    #Scaling
    testXAdd=testX[["ILD01In","ILD12In","ILD23In"]]
    testX=testX[["mA","mB","mC","subst"]]
    trainX=trainX[["mA","mB","mC","subst"]]
    scaler=StandardScaler()
    scaler.fit(trainX)
    trainX=scaler.transform(trainX)
    testX=scaler.transform(testX)
    xgBoost=XGBRegressor()
    xgBoostSet={'n_estimators': [5000]}
    xgBoost.fit(trainX,trainY)
    pred=xgBoost.predict(testX)
    return testY,pred,mean_absolute_error(testY,pred),testX,testXAdd
    
#Define target quantities
targ=["ILD01","ILD12","ILD23","mmA","mmB","mmC"]
end=firstBatch
df = pd.DataFrame(columns = ["Quantity","Amount of Data","Error Pred", "Error Guess"])
dfPand=dfPand.sample(n=firstBatch,random_state=np.random.RandomState(randomSeed))
for i in range(0,len(targ)):
    for k in np.arange(20,end,20):
        samp=dfPand.iloc[:k]
        teY,pred,mae,tex,add=trainBatch(samp[["mA","mB","mC","subst","ILD01In","ILD12In","ILD23In"]],samp[targ[i]])
        if i<3:
            dis=["ILD01In","ILD12In","ILD23In"]
            init=add[dis[i]]
        else:
            init=np.ones(np.shape(teY))
        print([targ[i],k,mae,mean_absolute_error(teY,init)])
        df = df._append({"Quantity":targ[i],"Amount of Data":int(k*0.8),"Error Pred":mae,"Error Guess":mean_absolute_error(teY,init)},ignore_index=True)

#Visualization of combined errors
df=df.replace({"ILD01": "$d_{AB}$","ILD12": "$d_{BC}$","ILD23": "$d_{CSub}$","mmA":"$m_A$" ,"mmB":"$m_B$" ,"mmC":"$m_C$" })
sns.set_theme(style="darkgrid")
sns.lineplot(data=df, x="Amount of Data", y="Error Pred", hue="Quantity")
plt.xlabel("Amount of training data")
plt.savefig("erorrsPred.png",dpi=1200)
plt.clf()

#Visualization of guess errors for magnetic moments
sns.set_theme(style="whitegrid")
sns.lineplot(data=df.query("Quantity=='$m_A$' |Quantity=='$m_B$' |Quantity=='$m_C$'"), x="Amount of Data", y="Error Guess", hue="Quantity")
plt.legend(ncol=1,title="Quantity")
plt.ylabel("MAE Guess [$\\mu_B$]")
plt.xlabel("Amount of Training Data")
plt.tight_layout()
plt.savefig("erorrGuess.png",dpi=1200)
plt.clf()

#Visualization of relation of guess errors for ILDs
sns.set_theme(style="whitegrid")
sns.lineplot(data=df.query("Quantity=='$d_{AB}$' |Quantity=='$d_{BC}$' |Quantity=='$d_{CSub}$'"), x="Amount of Data", y="Error Guess", hue="Quantity")
plt.legend(ncol=1,title="Quantity")
plt.ylabel("MAE Guess [$\\AA$]")
plt.xlabel("Amount of Training Data")
plt.tight_layout()
plt.savefig("erorrGuessDis.png",dpi=1200)
plt.clf()

#Visualization of relation of guess and prediction errors
df["$\\frac{MAE_{ML Pred}}{MAE_{Guess}}$"]=df["Error Pred"]/df["Error Guess"]
sns.set_theme(style="whitegrid")
sns.lineplot(data=df, x="Amount of Data", y="$\\frac{MAE_{ML Pred}}{MAE_{Guess}}$", hue="Quantity")
plt.legend(ncol=2,title="Quantity")
plt.xlabel("Amount of Training Data")
plt.ylim(0,1.5)
plt.axhline(y=1, color="r",linestyle="--")
plt.ylabel("$\\dfrac{MAE_{\\mathrm{DFTInt\ ML}}}{MAE_{\\mathrm{Guess}}}$")
plt.tight_layout()
plt.savefig("erorrsRelation.png",dpi=1200)


