#Some small functions which have been used for analytics in the course of this project.
#- Robin Hilgers, 2023
#Published on Zenodo
#--------------------------------------------------------------------------------
# Copyright (c) 2023 Peter Grünberg Institut, Forschungszentrum Jülich, Germany
# This file is available as free software under the conditions
# of the MIT license as expressed in the LICENSE file in more detail.
#--------------------------------------------------------------------------------

#Imports
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import statsmodels.formula.api as smf
import seaborn as sns 

#Fit quantiles
def fit_model(q,quant,labX):
        res = quant.fit(q=q)
        return [q, res.params["Intercept"], res.params[labX]] + res.conf_int().loc[
            labX].tolist()

##Make fancy plots all in one with different quantiles
def quantRegLinFitMargHist(df,labX,labY,saveName,quantStart,quantStop,quantSteps):
    quant = smf.quantreg(labY+" ~ "+labX,df)
    qReg = quant.fit(q=0.5)
    print("Quant reg q=0.5:")
    print(qReg.summary())
    quantiles = np.arange(quantStart,quantStop,quantSteps)#0.05,0.96,0.1

    models = [fit_model(x,quant,labX) for x in quantiles]
    models = pd.DataFrame(models, columns=["q", "a", "b", "lb", "ub"])
    ols = smf.ols(labY+" ~ "+labX, df).fit()
    ols_ci = ols.conf_int().loc[labX].tolist()
    ols = dict(
            a=ols.params["Intercept"], b=ols.params[labX], lb=ols_ci[0], ub=ols_ci[1]
    )
    xols = np.arange(df[labX].min(), df[labX].max(),(df[labX].max()-df[labX].min())/ 50)
    get_y = lambda a, b: a + b * xols
    f, ax = plt.subplots(figsize=(8, 8))
    sns.set_theme(style="whitegrid")
    g1=sns.jointplot(data=df,x=labX,y=labY,dropna = True,space = 0)
    g1.ax_joint.plot([0,np.max(df[labX])], [0,np.max(df[labX])], 'r-', linewidth = 2)
    for i in range(models.shape[0]):
            yols = get_y(models.a[i], models.b[i])
            g1.ax_joint.plot(xols, yols, "k-.",linewidth = 1,alpha=0.2,markersize=1)
    yols = get_y(ols["a"], ols["b"])
    g1.ax_joint.plot(xols, yols, "b-", label="OLS",linewidth = 2)
    xstd=np.std(df[labX])
    xmean=np.mean(df[labX])
    ystd=np.std(df[labY])
    ymean=np.mean(df[labY])
    g1.ax_marg_x.axvline(xmean, color='red', ls='--')
    g1.ax_marg_x.axvspan(xmean - xstd, xmean + xstd, color='red', alpha=0.1)
    g1.ax_marg_y.axhline(ymean, color='red', ls='--')
    g1.ax_marg_y.axhspan(ymean - ystd, ymean + ystd, color='red', alpha=0.1)
    plt.savefig(saveName+".png",dpi=600)
    plt.clf()
