#Collection of functions used to classify magnetic pair interactions and ordering using the magnetic moments of neigboring layers
#- Robin Hilgers, 2023
#Published on Zenodo
#--------------------------------------------------------------------------------
# Copyright (c) 2023 Peter Grünberg Institut, Forschungszentrum Jülich, Germany
# This file is available as free software under the conditions
# of the MIT license as expressed in the LICENSE file in more detail.
#--------------------------------------------------------------------------------

#Imports
import numpy as np
import classMagn

#Classify magnetic ordering into "Non Magnetic", "Ferro", "Ferri", "Anti Ferro"
def classMagn(mA,mB,mC):
	cutoff=0.4#Motiv durch Subst Boxplot -> Filter induced moments from magn layers
	mArr=np.array([mA,mB,mC])
	mArrO=np.array([mA,mB,mC])
	mArr=mArr[np.where(np.abs(mArr)>=cutoff)]
	print(mArr)
	if len(mArr)==0: return "No Mag"
	if (np.all(mArr>0) or np.all(mArr<0)): return "Ferro"
	elif len(mArr)==2: 
		if np.prod(mArr)<0: return "Anti Ferro"
	elif len(mArr)==3:
		if mArr[0]>0 and mArr[2]>0 and mArr[1]<0: return "Anti Ferro"
		if mArr[0]<0 and mArr[2]<0 and mArr[1]>0: return "Anti Ferro" 
		if np.sum(np.abs(mArrO))>np.sum(mArrO):  
			return "Ferri"
	print("Shouldn't be here:")


#Classify magnetic coupling
def couplingNoMag(mA,mB,mC):
	cutoff=0.4 #Motiv durch Subst Boxplot -> Filter induced moments from magn layers
	if np.all(np.array([np.abs(mA),np.abs(mB),np.abs(mC)])<cutoff):return
	mArr=np.array([mA,mB,mC])
	mArrLow=np.argwhere(np.abs(mArr)<cutoff)
	mArrHigh=np.argwhere(np.abs(mArr)>=cutoff)
	if (mArr[0]>0 and mArr[1]<0) or (mArr[0]<0 and mArr[1]>0): 
		copula=-1
	if (mArr[0]>0 and mArr[1]>0) or  (mArr[0]<0 and mArr[1]<0) : 
		copula=1
	if (mArr[1]>0 and mArr[2]>0) or  (mArr[1]<0 and mArr[2]<0) : 
		copulb=1
	if (mArr[1]>0 and mArr[2]<0) or (mArr[1]<0 and mArr[2]>0): 
		copulb=-1
	if mArr[1]==0 or mArr[2]==0: return
	if mArr[0]==0 or mArr[1]==0: return
	if len(mArrLow)==0: return
	print(mArrLow)
	ax=None
	bx=None
	cx=None
	for i in mArrLow:
		if i==0: ax="A"
		if i==1: bx="B"
		if i==2: cx="C"
	if len(mArrHigh)==2:
		if 0 in mArrLow:
			magnNeighA="B"
			magnNeighB="No"		
		if 1 in mArrLow:
			magnNeighA="C"
			magnNeighB="A"
		
		if 2 in mArrLow:
			magnNeighA="B"
			magnNeighB="No"
	elif len(mArrHigh)==1:
		for i in mArrHigh:
			if i==0: 
				magnNeighA="A"
				magnNeighB="No"
			if i==1: 
				magnNeighA="B"
				magnNeighB="No"
			if i==2: 
				magnNeighA="C"
				magnNeighB="No"
	if len(mArrHigh)==0:
		magnNeighA="No"
		magnNeighB="No"
	return [copula,copulb,ax,bx,cx,magnNeighA,magnNeighB]
