#Small script making a graphical representation of computed films. 
#- Robin Hilgers, 2023
#Published on Zenodo
#--------------------------------------------------------------------------------
# Copyright (c) 2023 Peter Grünberg Institut, Forschungszentrum Jülich, Germany
# This file is available as free software under the conditions
# of the MIT license as expressed in the LICENSE file in more detail.
#--------------------------------------------------------------------------------

#Imports
from ete3 import Tree
from ete3.treeview.main import TreeStyle
import pandas as pd
import os 
os.environ["QT_QPA_PLATFORM"]="offscreen"
def check_xvfb():
    comm = 'ps -ef | grep xvfb-run | grep %s | grep -v grep | cat' %os.path.splitext(os.path.basename(sys.argv[0]))[0]
    output = subprocess.check_output(comm, shell=True)

    if not len(output): 
        print ('script not called properly, retrying...')
        comm_run = 'xvfb-run ' + ' '.join(sys.argv)
        os.system(comm_run)
        sys.exit(0)
    else:
        print ('script called properly!')


df=pd.read_csv("../Data/data.csv")[["subst","mC","mB","mA"]].astype(int)
df["mC"]=df["subst"].astype(str)+df["mC"].astype(str)
df["mB"]=df["mC"].astype(str)+df["mB"].astype(str)
df["mA"]=+df["mB"].astype(str)+df["mA"].astype(str)



uni=df['subst'].unique()
node_to_children = {}
for row in df.itertuples():
    row_list = [element for element in row[1:] if element != ""]
    for i in range(len(row_list)-1):
        if row_list[i] in node_to_children.keys():
            if row_list[i+1] in node_to_children[row_list[i]].keys():
                continue
            else:
                node_to_children[row_list[i]].update({row_list[i+1]:0})
        else:
            node_to_children[row_list[i]] = {row_list[i+1]:0}
            
def newickify(node_to_children, root_node) -> str:
    visited_nodes = set()

    def newick_render_node(name, distance: float) -> str:
        assert name not in visited_nodes, "Error: The tree may not be circular!"

        if name not in node_to_children:
            # Leafs
            return F'{name}:{distance}'
        else:
            # Nodes
            visited_nodes.add(name)
            children = node_to_children[name]
            children_strings = [newick_render_node(child, children[child]) for child in children.keys()]
            children_strings = ",".join(children_strings)
            return F'({children_strings}){name}:{distance}'

    newick_string = newick_render_node(root_node, 0) + ';'
    return newick_string
    
for x in uni:
    print(node_to_children)
    newStr=newickify(node_to_children, x)
    print(newStr)
    t = Tree(newStr)
    ts = TreeStyle()
    ts.show_leaf_name = True
    ts.mode = "c"
    ts.arc_start = -180
    ts.arc_span = 360

    t.render("convTree"+str(int(x))+".png", w=250, units="mm", tree_style=ts)
