#This script fits a model function to the induced substrate moments and compiles corresponding visualizations
#Also the magnetic pair interaction between magnetic layers is examined
#- Robin Hilgers, 2023
#Published on Zenodo
#--------------------------------------------------------------------------------
# Copyright (c) 2023 Peter Grünberg Institut, Forschungszentrum Jülich, Germany
# This file is available as free software under the conditions
# of the MIT license as expressed in the LICENSE file in more detail.
#--------------------------------------------------------------------------------

#Imports
import numpy as np
import pandas as pd
import seaborn as sns
from sklearn.metrics import r2_score
import scipy 
import mendeleev as me
import matplotlib.pyplot as plt
from scipy.stats import norm

#Load data
df=pd.read_csv("../Data/data.csv")
#Only use films with three magnetic layers
df=df.query("numAtTypes==6")#Comment out for complete Heatmaps

#Choose which magnetic configuration should be exmained:
inMomPerSubst=pd.DataFrame(data=df, columns=["subst","mmSubA","mmSubB","mmSubC"]).abs()
#inMomPerSubst=pd.DataFrame(data=df, columns=["subst","mmSubA","mmSubB","mmSubC"]).query("(mmSubA>0 & mmSubB>0 & mmSubC>0)|(mmSubA<0 & mmSubB<0 & mmSubC<0)")
#inMomPerSubst=pd.DataFrame(data=df, columns=["subst","mmSubA","mmSubB","mmSubC"]).query("(mmSubA>0 & mmSubB>0 & mmSubC>0)")#.abs()

#Choose which magnetic configuration should be exmained:
ILDs=pd.DataFrame(data=df, columns=["subst","ILD01","ILD12","ILD23","ILD34"]).abs().groupby(by="subst").mean()
#ILDs=pd.DataFrame(data=df.query("(mmSubA>0 & mmSubB>0 & mmSubC>0)|(mmSubA<0 & mmSubB<0 & mmSubC<0)"), columns=["subst","ILD23","ILD34"]).abs().groupby(by="subst").mean()
#ILDs=pd.DataFrame(data=df.query("(mmSubA>0 & mmSubB>0 & mmSubC>0)"), columns=["subst","ILD23","ILD34"]).abs().groupby(by="subst").mean()

inMomPerSubstStd=pd.DataFrame(data=df, columns=["subst","mmSubA","mmSubB","mmSubC"]).abs()

#Sort data regarding substrate moment
inMomPerSubst=inMomPerSubst.rename(columns={"subst": "Substrate Element"})
inMomPerSubstStd=inMomPerSubstStd.rename(columns={"subst": "Substrate Element"})
inMomPerSubst=inMomPerSubst.rename(columns={"mmSubA":"Substrate A","mmSubB":"Substrate B","mmSubC":"Substrate C"})
inMomPerSubstStd=inMomPerSubstStd.rename(columns={"mmSubA":"Substrate A","mmSubB":"Substrate B","mmSubC":"Substrate C"})
inMomPerSubst=inMomPerSubst.replace(to_replace=[45.0,46.0,47.0,77.0,78.0,79.0],value=["Rh","Pd","Ag","Ir","Pt","Au"], regex=True)
inMomPerSubstStd=inMomPerSubstStd.replace(to_replace=[45.0,46.0,47.0,77.0,78.0,79.0],value=["Rh","Pd","Ag","Ir","Pt","Au"], regex=True)
unSbu=pd.melt(inMomPerSubst,id_vars='Substrate Element',value_vars=["Substrate A","Substrate B","Substrate C"],var_name='Layer', value_name='Magnetic Moment').groupby(by=["Layer","Substrate Element"]).mean()	
unSbuStd=pd.melt(inMomPerSubstStd,id_vars='Substrate Element',value_vars=["Substrate A","Substrate B","Substrate C"],var_name='Layer', value_name='Magnetic Moment').groupby(by=["Layer","Substrate Element"]).std()	

#Results average substrate moment magnitude per layer
au=unSbu.query("`Substrate Element`=='Au'").values
au=np.append(au,au[1])
au=np.append(au,au[0])
ir=unSbu.query("`Substrate Element`=='Ir'").values
ir=np.append(ir,ir[1])
ir=np.append(ir,ir[0])
rh=unSbu.query("`Substrate Element`=='Rh'").values
rh=np.append(rh,rh[1])
rh=np.append(rh,rh[0])
ag=unSbu.query("`Substrate Element`=='Ag'").values
ag=np.append(ag,ag[1])
ag=np.append(ag,ag[0])
pD=unSbu.query("`Substrate Element`=='Pd'").values
pD=np.append(pD,pD[1])
pD=np.append(pD,pD[0])
pt=unSbu.query("`Substrate Element`=='Pt'").values
pt=np.append(pt,pt[1])
pt=np.append(pt,pt[0])

#z corrdinates of substrate layers for each substrate element
xau=[ILDs.query("subst==79")["ILD23"].values[0],ILDs.query("subst==79")["ILD34"].values[0]+ILDs.query("subst==79")["ILD23"].values[0],ILDs.query("subst==79")["ILD23"].values[0]+2*ILDs.query("subst==79")["ILD34"].values[0],ILDs.query("subst==79")["ILD23"].values[0]+3*ILDs.query("subst==79")["ILD34"].values[0],ILDs.query("subst==79")["ILD23"].values[0]+4*ILDs.query("subst==79")["ILD34"].values[0]]#79
xau=xau-xau[2]
xag=[ILDs.query("subst==47")["ILD23"].values[0],ILDs.query("subst==47")["ILD34"].values[0]+ILDs.query("subst==47")["ILD23"].values[0],ILDs.query("subst==47")["ILD23"].values[0]+2*ILDs.query("subst==47")["ILD34"].values[0],ILDs.query("subst==47")["ILD23"].values[0]+3*ILDs.query("subst==47")["ILD34"].values[0],ILDs.query("subst==47")["ILD23"].values[0]+4*ILDs.query("subst==47")["ILD34"].values[0]]#47
xag=xag-xag[2]
xir=[ILDs.query("subst==77")["ILD23"].values[0],ILDs.query("subst==77")["ILD34"].values[0]+ILDs.query("subst==77")["ILD23"].values[0],ILDs.query("subst==77")["ILD23"].values[0]+2*ILDs.query("subst==77")["ILD34"].values[0],ILDs.query("subst==77")["ILD23"].values[0]+3*ILDs.query("subst==77")["ILD34"].values[0],ILDs.query("subst==77")["ILD23"].values[0]+4*ILDs.query("subst==77")["ILD34"].values[0]]#77
xir=xir-xir[2]
xrh=[ILDs.query("subst==45")["ILD23"].values[0],ILDs.query("subst==45")["ILD34"].values[0]+ILDs.query("subst==45")["ILD23"].values[0],ILDs.query("subst==45")["ILD23"].values[0]+2*ILDs.query("subst==45")["ILD34"].values[0],ILDs.query("subst==45")["ILD23"].values[0]+3*ILDs.query("subst==45")["ILD34"].values[0],ILDs.query("subst==45")["ILD23"].values[0]+4*ILDs.query("subst==45")["ILD34"].values[0]]#45
xrh=xrh-xrh[2]
xpD=[ILDs.query("subst==46")["ILD23"].values[0],ILDs.query("subst==46")["ILD34"].values[0]+ILDs.query("subst==46")["ILD23"].values[0],ILDs.query("subst==46")["ILD23"].values[0]+2*ILDs.query("subst==46")["ILD34"].values[0],ILDs.query("subst==46")["ILD23"].values[0]+3*ILDs.query("subst==46")["ILD34"].values[0],ILDs.query("subst==46")["ILD23"].values[0]+4*ILDs.query("subst==46")["ILD34"].values[0]]#46
xpD=xpD-xpD[2]
xpt=[ILDs.query("subst==78")["ILD23"].values[0],ILDs.query("subst==78")["ILD34"].values[0]+ILDs.query("subst==78")["ILD23"].values[0],ILDs.query("subst==78")["ILD23"].values[0]+2*ILDs.query("subst==78")["ILD34"].values[0],
ILDs.query("subst==78")["ILD23"].values[0]+3*ILDs.query("subst==78")["ILD34"].values[0],
ILDs.query("subst==78")["ILD23"].values[0]+4*ILDs.query("subst==78")["ILD34"].values[0]]#78
xpt=xpt-xpt[2]

#Fitted exponential function
def expo(t,a,b,c,d,psi,subWidth,noMagLay):#psi phase
	psi=0
	if c!=0 :return a*(np.exp(-b*(t+subWidth/2))+np.exp(-b*(subWidth/2-t)))*(1+c*(np.cos((t)*(d*2*noMagLay/(2*np.pi))**-1-psi)))
	else: return a*(np.exp(-b*(t+subWidth/2))+np.exp(-b*(subWidth/2-t)))#*(1+c*(np.sin((t-(subWidth-4*noMagLay)/2)*(d*2*noMagLay/(2*np.pi))**-1-psi)))

#Function to determine the period
def peri(d,noMagLay):
	if d==0: return 0
	else: 
		w=(d*2*noMagLay/(2*np.pi))**-1
		w=w/(2*np.pi)
		T=w**-1 
		print("Period: "+ str(T)+' A')
		print("Period: "+ str(T/noMagLay)+' Lay') 
		return
		
#Individual fits	
subWidthAg=2*ILDs.query("subst==47")["ILD23"].values[0]+4*ILDs.query("subst==47")["ILD34"].values[0]
resa,resba=scipy.optimize.curve_fit(lambda x,a,b,c,d:expo(x,a,b,c,d,0,subWidthAg,ILDs.query("subst==47")["ILD34"].values[0]),xag,ag,p0=(0.2,0.1,0.01,0.1))
peri(resa[3],ILDs.query("subst==47")["ILD34"].values[0])

subWidthAu=2*ILDs.query("subst==79")["ILD23"].values[0]+4*ILDs.query("subst==79")["ILD34"].values[0]
resb,resbb=scipy.optimize.curve_fit(lambda x,a,b,c,d:expo(x,a,b,c,d,0,subWidthAu,ILDs.query("subst==79")["ILD34"].values[0]),xau,au,p0=(0.2,0.1,0.01,0.1))
peri(resb[3],ILDs.query("subst==79")["ILD34"].values[0])

subWidthIr=2*ILDs.query("subst==77")["ILD23"].values[0]+4*ILDs.query("subst==77")["ILD34"].values[0]
resc,resbc=scipy.optimize.curve_fit(lambda x,a,b,c,d,psi:expo(x,a,b,c,d,psi,subWidthIr,ILDs.query("subst==77")["ILD34"].values[0]),xir,ir,p0=(0.2,0.1,0.0001,2,0.0))
peri(resc[3],ILDs.query("subst==77")["ILD34"].values[0])

subWidthRh=2*ILDs.query("subst==45")["ILD23"].values[0]+4*ILDs.query("subst==45")["ILD34"].values[0]
resd,resbd=scipy.optimize.curve_fit(lambda x,a,b,c,d,psi:expo(x,a,b,c,d,psi,subWidthRh,ILDs.query("subst==45")["ILD34"].values[0]),xrh,rh,p0=(0.2,0.3,0.5,1.5,0.2))
peri(resd[3],ILDs.query("subst==45")["ILD34"].values[0])

subWidthPd=2*ILDs.query("subst==46")["ILD23"].values[0]+4*ILDs.query("subst==46")["ILD34"].values[0]
resf,resbf=scipy.optimize.curve_fit(lambda x,a,b,c,d,psi:expo(x,a,b,c,d,psi,subWidthPd,ILDs.query("subst==46")["ILD34"].values[0]),xpD,pD,p0=(0.2,0.1,1,1.5,0.2))
peri(resf[3],ILDs.query("subst==46")["ILD34"].values[0])

subWidthPt=2*ILDs.query("subst==78")["ILD23"].values[0]+4*ILDs.query("subst==78")["ILD34"].values[0]
resg,resbg=scipy.optimize.curve_fit(lambda x,a,b,c,d:expo(x,a,b,c,d,0,subWidthPt,ILDs.query("subst==78")["ILD34"].values[0]),xpt,pt,p0=(0.2,0.1,0.1,1.5))
peri(resg[3],ILDs.query("subst==78")["ILD34"].values[0])

#Fitting results
basis=np.array([np.exp(-resa[1]),np.exp(-resb[1]),np.exp(-resc[1]),np.exp(-resd[1]),np.exp(-resf[1]),np.exp(-resg[1])])
amp=np.array([resa[0],resb[0],resc[0],resd[0],resf[0],resg[0]])
kLay=np.array([resa[3]*ILDs.query("subst==47")["ILD34"].values[0],resb[3]*ILDs.query("subst==79")["ILD34"].values[0],resc[3]*ILDs.query("subst==77")["ILD34"].values[0],resd[3]*ILDs.query("subst==45")["ILD34"].values[0],resf[3]*ILDs.query("subst==46")["ILD34"].values[0],resg[3]*ILDs.query("subst==78")["ILD34"].values[0]])
kFit=np.array([resa[3],resb[3],resc[3],resd[3],resf[3],resg[3]])
kOuter=np.array([resa[3]*ILDs.query("subst==47")["ILD23"].values[0],resb[3]*ILDs.query("subst==79")["ILD23"].values[0],resc[3]*ILDs.query("subst==77")["ILD23"].values[0],resd[3]*ILDs.query("subst==45")["ILD23"].values[0],resf[3]*ILDs.query("subst==46")["ILD23"].values[0],resg[3]*ILDs.query("subst==78")["ILD23"].values[0]])
kTotSub=np.array([resa[3]*subWidthAg,resb[3]*subWidthAu,resc[3]*subWidthIr,resd[3]*subWidthRh,resf[3]*subWidthPd,resg[3]*subWidthPt])

atoms=["Ag","Au","Ir","Rh","Pd","Pt"]
aF=np.zeros(0,dtype=int)

#Get atomic numbers
for i in range (0,len(atoms)):
	aF=np.append(aF,me.element(atoms[i]).atomic_number)

#Visualized fits
sns.set_theme(style="whitegrid")
palette=sns.color_palette("rocket",6)
plt.figure(figsize=(10,5))
plt.plot(np.arange(np.min(-subWidthPd/2),np.max(subWidthPd/2),0.01),expo(np.arange(np.min(-subWidthPd/2),np.max(subWidthPd/2),.01),resf[0],resf[1],resf[2],resf[3],0,subWidthPd,ILDs.query("subst==46")["ILD34"].values[0]),label="Pd",color=palette[0])
plt.plot(np.arange(np.min(-subWidthPt/2),np.max(subWidthPt/2),0.01),expo(np.arange(np.min(-subWidthPt/2),np.max(subWidthPt/2),.01),resg[0],resg[1],resg[2],resg[3],0,subWidthPt,ILDs.query("subst==78")["ILD34"].values[0]),label="Pt",color=palette[1])
plt.plot(np.arange(np.min(-subWidthRh/2),np.max(subWidthRh/2),0.01),expo(np.arange(np.min(-subWidthRh/2),np.max(subWidthRh/2),.01),resd[0],resd[1],resd[2],resd[3],0,subWidthRh,ILDs.query("subst==45")["ILD34"].values[0]),label="Rh",color=palette[2])
plt.plot(np.arange(np.min(-subWidthIr/2),np.max(subWidthIr/2),0.01),expo(np.arange(np.min(-subWidthIr/2),np.max(subWidthIr/2),.01),resc[0],resc[1],resc[2],resc[3],resc[4],subWidthIr,ILDs.query("subst==77")["ILD34"].values[0]),label="Ir",color=palette[3])
plt.plot(np.arange(np.min(-subWidthAg/2),np.max(subWidthAg/2),0.01),expo(np.arange(np.min(-subWidthAg/2),np.max(subWidthAg/2),.01),resa[0],resb[1],resa[2],resa[3],0,subWidthAg,ILDs.query("subst==47")["ILD34"].values[0]),label="Ag",color=palette[5])
plt.plot(np.arange(np.min(-subWidthAu/2),np.max(subWidthAu/2),0.01),expo(np.arange(np.min(-subWidthAu/2),np.max(subWidthAu/2),.01),resb[0],resb[1],resb[2],resb[3],0,subWidthAu,ILDs.query("subst==79")["ILD34"].values[0]),label="Au",color=palette[4])

plt.scatter(xpD,pD,label="Pd",color=palette[0],marker="s")
plt.scatter(xpt,pt,label="Pt",color=palette[1],marker="s")
plt.scatter(xrh,rh,label="Rh",color=palette[2],marker="s")
plt.scatter(xir,ir,label="Ir",color=palette[3],marker="s")
plt.scatter(xag,ag,label="Ag",color=palette[5],marker="s")
plt.scatter(xau,au,label="Au",color=palette[4],marker="s")
plt.xlabel("Distance Along the $z$-axis From the Substrate's Center [$\\AA$]")
plt.ylabel("Average Magnitude of the Induced \n Magnetic Substrate Moment [$\\mu_B$]")
plt.legend(ncols=4)
plt.ylim(0,0.15)
plt.legend(loc="center left",bbox_to_anchor=(1,0.5))
plt.tight_layout()
plt.savefig("Fits.png",dpi=1200)
plt.clf()

#Plots basis and a,plidute
plt.scatter(aF,basis,label="Basis")
plt.scatter(aF,amp,label="Amp")
plt.legend()
plt.xlabel("Atom Number")
plt.savefig("BasisAmp.png",dpi=800)
plt.clf()

#EDA regarding the results and the atomic numbers
plt.scatter(aF,(kFit**-1)*(np.pi*2),label="Period From Fit")
plt.scatter(aF,np.array([subWidthAg,subWidthAu,subWidthIr,subWidthRh,subWidthPd,subWidthPt]),label="Substrate Thickness with C-Sub distance")
plt.scatter(aF,np.array([subWidthAg-ILDs.query("subst==47")["ILD23"].values[0]*2,subWidthAu-ILDs.query("subst==79")["ILD23"].values[0]*2,subWidthIr-ILDs.query("subst==77")["ILD23"].values[0]*2,subWidthRh-ILDs.query("subst==45")["ILD23"].values[0]*2,subWidthPd-ILDs.query("subst==46")["ILD23"].values[0]*2,subWidthPt-ILDs.query("subst==78")["ILD23"].values[0]*2]),label="Substrate Thickness without C-Sub distance")
totK=np.array([subWidthAg+ILDs.query("subst==47")["ILD12"].values[0]*2+ILDs.query("subst==47")["ILD01"].values[0]*2,subWidthAu+ILDs.query("subst==79")["ILD12"].values[0]*2+ILDs.query("subst==79")["ILD01"].values[0]*2,subWidthIr+ILDs.query("subst==77")["ILD12"].values[0]*2+ILDs.query("subst==77")["ILD01"].values[0]*2,subWidthRh+ILDs.query("subst==45")["ILD12"].values[0]*2+ILDs.query("subst==45")["ILD01"].values[0]*2,subWidthPd+ILDs.query("subst==46")["ILD12"].values[0]*2+ILDs.query("subst==46")["ILD01"].values[0]*2,subWidthPt+ILDs.query("subst==78")["ILD12"].values[0]*2+ILDs.query("subst==78")["ILD01"].values[0]*2])
plt.scatter(aF,totK,label="Total thickness")
plt.scatter(aF,np.array([subWidthAg,subWidthAu,subWidthIr,subWidthRh,subWidthPd,subWidthPt])*2.5/7,label="Substrate Thickness with C-Sub distance per layer x 2.5")
plt.scatter(aF,totK/11,label="Total thickness per layer ")
plt.scatter(aF,2.5*totK/11,label="Total thickness per layer x2.5")
plt.ylabel("Distance in $\\AA$")
plt.legend()
plt.xlabel("Atomic Number")
plt.savefig("At#K.png",dpi=800)
plt.clf()
aF=np.zeros(0,dtype=int)

#EDA regarding the results and the valence electron numbers
for i in range (0,len(atoms)):
	aF=np.append(aF,me.element(atoms[i]).nvalence())
print(aF)
plt.scatter(aF,basis,label="Basis")
plt.scatter(aF,amp,label="Amp")
plt.legend()
plt.xlabel("Valence El Number")
plt.savefig("BasisValElAmp.png",dpi=800)
plt.clf()

#Magnetic moment development in substrate progression
unSbu=unSbu.rename(columns={ "Magnetic Moment": "Magnetic Moment in $\\mu_B$"})
palette = sns.color_palette("rocket", 6)
palette2 = sns.color_palette("rocket", 6)
paletteCorr=palette
paletteCorr[4]=palette[5]
paletteCorr[5]=palette2[4]
sns.set_style(style="whitegrid")
sns.lineplot(data=unSbu, x="Layer", y="Magnetic Moment in $\\mu_B$", hue="Substrate Element",hue_order=["Pd","Pt","Rh","Ir","Ag","Au"],palette=paletteCorr)
plt.legend(bbox_to_anchor =(0.5,1.05), loc='lower center',ncols=6)
plt.ylabel("Average Magnitude of the Magnetic \n Substrate Moment in $\\mu_B$ per Layer")
plt.xlabel("Substrate Layer")
plt.xticks(ticks=[0,1,2],labels=["A","B","C"])
plt.tight_layout()
plt.savefig("MagnMomentDevelSubstrate.png",dpi=1200)
plt.clf()

#Pair interaction types for magnetic layers
mmLayNoSub=pd.DataFrame(data=df, columns=["mA","mB","mC","subst","mmA","mmB","mmC"])
mmLayNoSub=mmLayNoSub.rename(columns={"mA":"$Z_A$","mB":"$Z_B$","mC":"$Z_C$","subst":"$Z_{Sub}$","mmA":"$m_A$","mmB":"$m_B$","mmC":"$m_C$"})
aMag=np.array(mmLayNoSub["$m_A$"].values)
bMag=np.array(mmLayNoSub["$m_B$"].values)
cMag=np.array(mmLayNoSub["$m_C$"].values)
abMag=aMag*bMag
bcMag=bMag*cMag
ab=np.zeros(0,dtype=int)
bc=np.zeros(0,dtype=int)
thres=0
for i in range (0,len(abMag)):
	if (np.abs(aMag[i])< thres and np.abs(bMag[i])<thres) or bMag[i]==0 or aMag[i]==0:
		ab=np.append(ab,0)	
	else:
		if abMag[i]>0:
			ab=np.append(ab,1)
		if abMag[i]<0:
			ab=np.append(ab,-1)	
	if np.abs(bMag[i])< thres and np.abs(cMag[i])<thres or  bMag[i]==0 or cMag[i]==0:
		bc=np.append(bc,0)	
	else:
		if bcMag[i]>0:
			bc=np.append(bc,1)
		if bcMag[i]<0:
			bc=np.append(bc,-1)	

#Average magnetism interaction AB per elements
mmLayNoSub=mmLayNoSub.reset_index()
mmLayNoSub["ABPairMag"]=pd.DataFrame(ab)
mmLayNoSub["BCPairMag"]=pd.DataFrame(bc)
piv=pd.pivot_table(mmLayNoSub,values=["ABPairMag"],index=["$Z_A$"],columns=["$Z_B$"],aggfunc=np.mean)
sns.heatmap(piv,cmap="icefire")
plt.yticks(ticks=np.flip([1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5,10.5]),labels=np.flip(["Sc","Ti","V","Cr","Mn","Fe","Co","Ni","Cu","Zn"]),rotation=0)
plt.xticks(ticks=np.flip([1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5,10.5]),labels=np.flip(["Sc","Ti","V","Cr","Mn","Fe","Co","Ni","Cu","Zn"]),rotation=0)
plt.ylabel("Magnetic A Layer Element")
plt.ylim(1,11)
sns.set_style("ticks")
plt.xlim(1,11)
plt.xlabel("Magnetic B Layer Element")
plt.tight_layout()
plt.savefig("HeatEleABNoSub.png",dpi=1200)
plt.clf()

##Average magnetism interaction BC per elements
piv=pd.pivot_table(mmLayNoSub,values=["BCPairMag"],index=["$Z_B$"],columns=["$Z_C$"],aggfunc=np.mean)
sns.heatmap(piv,cmap="icefire")
plt.yticks(ticks=np.flip([1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5,10.5]),labels=np.flip(["Sc","Ti","V","Cr","Mn","Fe","Co","Ni","Cu","Zn"]),rotation=0)
plt.xticks(ticks=np.flip([0.5,1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]),labels=np.flip(["Sc","Ti","V","Cr","Mn","Fe","Co","Ni","Cu","Zn"]),rotation=0)
sns.set_style("ticks")
plt.ylim(1,11)
plt.ylabel("Magnetic B Layer Element")
plt.xlabel("Magnetic C Layer Element")
plt.tight_layout()
plt.savefig("HeatEleBCNoSub.png",dpi=1200)
plt.clf()

#Number of ferromagnetic interactions AB per elements
dataOccupF=mmLayNoSub.loc[(mmLayNoSub["ABPairMag"]>0)]
pivF=pd.pivot_table(dataOccupF,values=["ABPairMag"],index=["$Z_A$"],columns=["$Z_B$"],aggfunc={"ABPairMag":"count"})
sns.heatmap(pivF,cmap="icefire")#,"BCPairMag"
plt.yticks(ticks=np.flip([0.5,1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]),labels=np.flip(["Sc","Ti","V","Cr","Mn","Fe","Co","Ni","Cu","Zn"]),rotation=0)
plt.xticks(ticks=np.flip([0.5,1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]),labels=np.flip(["Sc","Ti","V","Cr","Mn","Fe","Co","Ni","Cu","Zn"]),rotation=0)
plt.ylabel("Magnetic A Layer Element")
plt.xlabel("Magnetic B Layer Element")
sns.set_style("ticks")
plt.tight_layout()
plt.savefig("HeatEleNoSubABF.png",dpi=1200)
plt.clf()

#Number of non magnetic interactions AB per elements
dataOccupNN=mmLayNoSub.loc[(mmLayNoSub["ABPairMag"]==0)]
pivNN=pd.pivot_table(dataOccupNN,values=["ABPairMag"],index=["$Z_A$"],columns=["$Z_B$"],aggfunc={"ABPairMag":"count"})
sns.heatmap(pivNN,cmap="icefire")
plt.yticks(ticks=np.flip([1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5,10.5]),labels=np.flip(["Sc","Ti","V","Cr","Mn","Fe","Co","Ni","Cu","Zn"]),rotation=0)
plt.xticks(ticks=np.flip([1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5,10.5]),labels=np.flip(["Sc","Ti","V","Cr","Mn","Fe","Co","Ni","Cu","Zn"]),rotation=0)
plt.ylabel("Magnetic A Layer Element")
plt.ylim(1,11)
plt.xlim(1,11)
sns.set_style("ticks")
plt.xlabel("Magnetic B Layer Element")
plt.tight_layout()
plt.savefig("HeatEleNoSubABNN.png",dpi=1200)
plt.clf()

#Number of anti-ferromagnetic interactions AB per elements
dataOccupAFM=mmLayNoSub.loc[(mmLayNoSub["ABPairMag"]<0)]
pivAFM=pd.pivot_table(dataOccupAFM,values=["ABPairMag"],index=["$Z_A$"],columns=["$Z_B$"],aggfunc={"ABPairMag":"count"})
sns.heatmap(pivAFM,cmap="icefire")
plt.yticks(ticks=np.flip([0.5,1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]),labels=np.flip(["Sc","Ti","V","Cr","Mn","Fe","Co","Ni","Cu","Zn"]),rotation=0)
plt.xticks(ticks=np.flip([0.5,1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]),labels=np.flip(["Sc","Ti","V","Cr","Mn","Fe","Co","Ni","Cu","Zn"]),rotation=0)
plt.ylabel("Magnetic A Layer Element")
plt.xlabel("Magnetic B Layer Element")
sns.set_style("ticks")
plt.tight_layout()
plt.savefig("HeatEleNoSubABAFM.png",dpi=1200)
plt.clf()

##Number of ferromagnetic interactions BC per elements
dataOccupF=mmLayNoSub.loc[(mmLayNoSub["BCPairMag"]>0)]
pivF=pd.pivot_table(dataOccupF,values=["BCPairMag"],index=["$Z_B$"],columns=["$Z_C$"],aggfunc={"BCPairMag":"count"})
sns.heatmap(pivF,cmap="icefire")#,"BCPairMag"
plt.yticks(ticks=np.flip([0.5,1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]),labels=np.flip(["Sc","Ti","V","Cr","Mn","Fe","Co","Ni","Cu","Zn"]),rotation=0)
plt.xticks(ticks=np.flip([0.5,1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]),labels=np.flip(["Sc","Ti","V","Cr","Mn","Fe","Co","Ni","Cu","Zn"]),rotation=0)
plt.ylabel("Magnetic B Layer Element")
plt.xlabel("Magnetic C Layer Element")
sns.set_style("ticks")
#plt.title("BC FM Count")
plt.tight_layout()
plt.savefig("HeatEleNoSubBCF.png",dpi=1200)
plt.clf()

#Number of non magnetic interactions BC per elements
dataOccupNN=mmLayNoSub.loc[(mmLayNoSub["BCPairMag"]==0)]
pivNN=pd.pivot_table(dataOccupNN,values=["BCPairMag"],index=["$Z_B$"],columns=["$Z_C$"],aggfunc={"BCPairMag":"count"})
sns.heatmap(pivNN,cmap="icefire")
plt.yticks(ticks=np.flip([1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5,10.5]),labels=np.flip(["Sc","Ti","V","Cr","Mn","Fe","Co","Ni","Cu","Zn"]),rotation=0)
plt.xticks(ticks=np.flip([0.5,1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]),labels=np.flip(["Sc","Ti","V","Cr","Mn","Fe","Co","Ni","Cu","Zn"]),rotation=0)
plt.ylabel("Magnetic B Layer Element")
plt.ylim(1,11)
sns.set_style("ticks")
plt.xlabel("Magnetic C Layer Element")
#plt.title("BC No Mag Count")
plt.tight_layout()
plt.savefig("HeatEleNoSubBCNN.png",dpi=1200)
plt.clf()

#Number of anti-ferromagnetic interactions BC per elements
dataOccupAFM=mmLayNoSub.loc[(mmLayNoSub["BCPairMag"]<0)]
pivAFM=pd.pivot_table(dataOccupAFM,values=["BCPairMag"],index=["$Z_B$"],columns=["$Z_C$"],aggfunc={"BCPairMag":"count"})
sns.heatmap(pivAFM,cmap="icefire")
plt.yticks(ticks=np.flip([0.5,1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]),labels=np.flip(["Sc","Ti","V","Cr","Mn","Fe","Co","Ni","Cu","Zn"]),rotation=0)
plt.xticks(ticks=np.flip([0.5,1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]),labels=np.flip(["Sc","Ti","V","Cr","Mn","Fe","Co","Ni","Cu","Zn"]),rotation=0)
plt.ylabel("Magnetic B Layer Element")
plt.xlabel("Magnetic C Layer Element")
sns.set_style("ticks")
plt.tight_layout()
plt.savefig("HeatEleNoSubBCAFM.png",dpi=1200)
plt.clf()

#AB pair interaction
dataOccup=pd.DataFrame(data=df, columns=["mA","mB","mC","mmSubA","mmSubB","mmSubC","totCellMom"])
aMag=np.array(dataOccup["mmSubA"].values)
bMag=np.array(dataOccup["mmSubB"].values)
cMag=np.array(dataOccup["mmSubC"].values)
abMag=aMag*bMag
bcMag=bMag*cMag
ab=np.zeros(0,dtype=int)
bc=np.zeros(0,dtype=int)
for i in range (0,len(abMag)):
	if abMag[i]>0:
		ab=np.append(ab,1)
	if abMag[i]<0:
		ab=np.append(ab,-1)	
	if abMag[i]==0:
		ab=np.append(ab,0)	
	if bcMag[i]>0:
		bc=np.append(bc,1)
	if bcMag[i]<0:
		bc=np.append(bc,-1)	
	if bcMag[i]==0:
		bc=np.append(bc,0)
dataOccup=dataOccup.reset_index()
dataOccup["ABPairMag"]=pd.DataFrame(ab)
dataOccup["BCPairMag"]=pd.DataFrame(bc)
piv=pd.pivot_table(dataOccup,values=["ABPairMag","BCPairMag"],index=["mA"],columns=["mB"],aggfunc=np.mean)
sns.heatmap(piv,cmap="icefire")
plt.tight_layout()
plt.savefig("HeatEleAB.png",dpi=600)
plt.clf()

###Count F, AFM, NM
dataOccupF=dataOccup.loc[(dataOccup["ABPairMag"]>0)]
pivF=pd.pivot_table(dataOccupF,values=["ABPairMag"],index=["mA"],columns=["mB"],aggfunc={"ABPairMag":"count"})
sns.heatmap(pivF,cmap="vlag")#,"BCPairMag"
plt.tight_layout()
plt.savefig("HeatEleABF.png",dpi=600)
plt.clf()

dataOccupNN=dataOccup.loc[(dataOccup["ABPairMag"]==0)]
pivNN=pd.pivot_table(dataOccupNN,values=["ABPairMag"],index=["mA"],columns=["mB"],aggfunc={"ABPairMag":"count"})
sns.heatmap(pivNN,cmap="vlag")
plt.tight_layout()
plt.savefig("HeatEleABNN.png",dpi=600)
plt.clf()

dataOccupAFM=dataOccup.loc[(dataOccup["ABPairMag"]<0)]
pivAFM=pd.pivot_table(dataOccupAFM,values=["ABPairMag"],index=["mA"],columns=["mB"],aggfunc={"ABPairMag":"count"})
sns.heatmap(pivAFM,cmap="vlag")
plt.tight_layout()
plt.savefig("HeatEleABAFM.png",dpi=600)
plt.clf()

dataOccupF=dataOccup.loc[(dataOccup["BCPairMag"]>0)]
pivF=pd.pivot_table(dataOccupF,values=["BCPairMag"],index=["mB"],columns=["mC"],aggfunc={"BCPairMag":"count"})
sns.heatmap(pivF,cmap="vlag")#,"BCPairMag"
plt.tight_layout()
plt.savefig("HeatEleABFBC.png",dpi=600)
plt.clf()

dataOccupNN=dataOccup.loc[(dataOccup["BCPairMag"]==0)]
pivNN=pd.pivot_table(dataOccupNN,values=["BCPairMag"],index=["mB"],columns=["mC"],aggfunc={"BCPairMag":"count"})
sns.heatmap(pivNN,cmap="vlag")
plt.tight_layout()
plt.savefig("HeatEleABNNBC.png",dpi=600)
plt.clf()

dataOccupAFM=dataOccup.loc[(dataOccup["BCPairMag"]<0)]
pivAFM=pd.pivot_table(dataOccupAFM,values=["BCPairMag"],index=["mB"],columns=["mC"],aggfunc={"BCPairMag":"count"})
sns.heatmap(pivAFM,cmap="vlag")
plt.tight_layout()
plt.savefig("HeatEleABAFMBC.png",dpi=600)
plt.clf()


sns.histplot(
	data=dataOccup,x="ABPairMag",
    edgecolor=".3",
    linewidth=.5,
    log_scale=False,stat="count"
)
plt.tight_layout()
plt.savefig("HistFMAFMPairAB.png",dpi=600)
plt.clf()

####Only considering magnetic Mag
df=df.loc[~((df[["mmA","mmB","mmC"]].abs()<0.1).any(axis=1))]
dataMag=pd.DataFrame(data=df, columns=["mA","mB","mC","mmSubA","mmSubB","mmSubC"])

aMag=np.array(dataMag["mmSubA"].values)
bMag=np.array(dataMag["mmSubB"].values)
cMag=np.array(dataMag["mmSubC"].values)
abMag=aMag*bMag
bcMag=bMag*cMag
ab=np.zeros(0,dtype=int)
bc=np.zeros(0,dtype=int)
for i in range (0,len(abMag)):
	if abMag[i]>0:
		ab=np.append(ab,1)
	if abMag[i]<0:
		ab=np.append(ab,-1)	
	if abMag[i]==0:
		ab=np.append(ab,0)	
	if bcMag[i]>0:
		bc=np.append(bc,1)
	if bcMag[i]<0:
		bc=np.append(bc,-1)	
	if bcMag[i]==0:
		bc=np.append(bc,0)
dataMag=dataMag.reset_index()
dataMag["ABPairMag"]=pd.DataFrame(ab)
dataMag["BCPairMag"]=pd.DataFrame(bc)
piv=pd.pivot_table(dataMag,values=["ABPairMag","BCPairMag"],index=["mA"],columns=["mB"],aggfunc=np.mean)
print(piv)
sns.heatmap(piv,cmap="vlag")
plt.tight_layout()
plt.savefig("HeatEleABOnlyMag.png",dpi=600)
plt.clf()

sns.histplot(
	data=dataOccup,x="BCPairMag",
    edgecolor=".3",
    linewidth=.5,
    log_scale=False,stat="count"
)
plt.tight_layout()
plt.savefig("HistFMAFMPairBC.png",dpi=600)
plt.clf()

#####BOXPLOT A
df=pd.read_csv("../Data/data.csv")

#Magnetic moment per substrate
dataOccup=pd.DataFrame(data=df, columns=["mA","mB","mC","mmC","mmB","mmA","subst","mmSubA","mmSubB","mmSubC"])
dataOccup["SubstAbs"]=2*(dataOccup.mmSubA.abs()+dataOccup.mmSubB.abs())+dataOccup.mmSubC.abs()
dataOccup["SubstNetM"]=2*(dataOccup.mmSubA+dataOccup.mmSubB)+dataOccup.mmSubC
dataOccup["TotMag"]=dataOccup["SubstNetM"]+2*(dataOccup["mmA"]+dataOccup["mmB"]+dataOccup["mmC"])
sns.set_theme(style="whitegrid")
sns.boxplot(x=dataOccup["subst"],y=dataOccup["SubstAbs"])
#plt.show()
plt.tight_layout()
plt.savefig("boxMMperSubst.png",dpi=600)
plt.clf()

#Net magnetic moment per substrate
sns.boxplot(x=dataOccup["subst"],y=dataOccup["SubstNetM"],meanprops={"marker":"o",
                       "markerfacecolor":"white", 
                       "markeredgecolor":"black",
                      "markersize":"6"},showmeans=True,palette=sns.color_palette("colorblind", 6))
plt.xlabel("Substrate Element")
plt.ylabel("$m^{\\mathrm{Tot}}_{Sub}$ in $\\mu_B$")
plt.xticks(labels=["Rh","Pd","Ag","Ir","Pt","Au"],ticks=[0,1,2,3,4,5])
plt.tight_layout()
plt.savefig("boxNetMMperSubst.png",dpi=1200)
plt.clf()

for i in ["mmSubA","mmSubB","mmSubC"]:
	if i=="mmSubA":
		lay="A"
	if i=="mmSubB":
		lay="B"
	if i=="mmSubC":
		lay="C"
	sns.set_theme(style="whitegrid")
	sns.boxplot(x=dataOccup["subst"],y=dataOccup[i],meanprops={"marker":"o",
                       "markerfacecolor":"white", 
                       "markeredgecolor":"black",
                      "markersize":"6"},showmeans=True,palette=sns.color_palette("colorblind", 6))

	plt.xlabel("Element in Substrate Layer "+ lay)
	plt.ylabel("$m_{Sub"+lay+"}$ in $\\mu_B$")
	plt.xticks(labels=["Rh","Pd","Ag","Ir","Pt","Au"],ticks=[0,1,2,3,4,5])
	plt.tight_layout()
	plt.tight_layout()
	plt.savefig("boxMMperSubstnet"+i+".png",dpi=1200)
	plt.clf()
	
##M's of M Lag
for i in ["mA","mB","mC"]:
	sns.set_theme(style="whitegrid")
	if i =="mA": 
		choice="mmA"
		lay="A"
		sns.boxplot(data=dataOccup.query("mA!=0"),x=i,y=choice,meanprops={"marker":"o",
                       "markerfacecolor":"white", 
                       "markeredgecolor":"black",
                      "markersize":"6"},showmeans=True,palette=sns.color_palette("colorblind", 10))
	if i== "mB": 
		choice="mmB"
		lay="B"
		sns.boxplot(data=dataOccup.query("mB!=0"),x=i,y=choice,meanprops={"marker":"o",
                       "markerfacecolor":"white", 
                       "markeredgecolor":"black",
                      "markersize":"6"},showmeans=True,palette=sns.color_palette("colorblind", 10))
	if i=="mC": 
		choice="mmC"
		lay="C"
		sns.boxplot(data=dataOccup,x=i,y=choice,meanprops={"marker":"o",
                       "markerfacecolor":"white", 
                       "markeredgecolor":"black",
                      "markersize":"6"},showmeans=True,palette=sns.color_palette("colorblind", 10))
	plt.xlabel("Magnetic Element in Layer "+ lay)
	plt.ylabel("$m_{"+lay+"}$ in $\\mu_B$")
	plt.xticks(labels=["Sc","Ti","V","Cr","Mn","Fe","Co","Ni","Cu","Zn"],ticks=[0,1,2,3,4,5,6,7,8,9])
	plt.tight_layout()
	plt.savefig("boxMMperElePerLayerNet"+i+".png",dpi=1200)
	plt.clf()

#TotMag per subst
sns.boxplot(x=df["subst"],y=df["totCellMom"],meanprops={"marker":"o",
                       "markerfacecolor":"white", 
                       "markeredgecolor":"black",
                      "markersize":"6"},showmeans=True,palette=sns.color_palette("colorblind", 6))
plt.xlabel("Substrate Element")
plt.ylabel("$M^{\\mathrm{Tot}}_{\\mathrm{Cell}}$ in $\\mu_B$")
plt.xticks(labels=["Rh","Pd","Ag","Ir","Pt","Au"],ticks=[0,1,2,3,4,5])
plt.tight_layout()
plt.savefig("boxTotMagperSubst.png",dpi=1200)
plt.clf()

dataOccup=dataOccup.dropna()

def norm_dist(mean,std,color="black"):
	x= np.linspace(mean-5*std,mean+5*std,200)
	p = norm.pdf(x,mean,std)
	z=plt.plot(x,p,color,linewidth=2) 


####BOXPLOT B
sns.set_theme(style="whitegrid")
sns.boxplot(data=dataOccup)
plt.savefig("boxMMSUB.png",dpi=600)
plt.clf()
dataOccup["AB"]=dataOccup["mmSubA"]*dataOccup["mmSubB"]
dataOccup["BC"]=dataOccup["mmSubC"]*dataOccup["mmSubB"]

lower=dataOccup.quantile(q=0.25, axis=0, numeric_only=False, interpolation='lower')
upper=dataOccup.quantile(q=0.75, axis=0, numeric_only=False, interpolation='higher')
lower=dataOccup.quantile(q=0.10, axis=0, numeric_only=False, interpolation='lower')
upper=dataOccup.quantile(q=0.90, axis=0, numeric_only=False, interpolation='higher')
lower=dataOccup.quantile(q=0.01, axis=0, numeric_only=False, interpolation='lower')
upper=dataOccup.quantile(q=0.99, axis=0, numeric_only=False, interpolation='higher')
maxi=dataOccup.max()
mini=dataOccup.min()


median=dataOccup.median()
mean=dataOccup.mean()
dataOccup=dataOccup[["mmSubA","mmSubB","mmSubC"]]
for i in dataOccup.columns:
	mu,std=norm.fit(dataOccup[i])
	sns.set_theme(style="ticks")
	f, ax = plt.subplots(figsize=(5, 5))
	sns.despine(f)
	sns.histplot(
	data=dataOccup[i],
    palette="light:m_r",
    edgecolor=".3",
    linewidth=.5,
    log_scale=False,stat="density"
)
	plt.xlim(-2,2)
	ax.set(xlabel='Magnetic Substrate Moment')
	norm_dist(mu,std)
	plt.tight_layout()
	plt.grid()
	sns.color_palette("blend:#7AB,#EDA", as_cmap=True)
	plt.savefig("Hist"+i+".png",dpi=1200)
	plt.clf()
	
	
